"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const dotenv_1 = __importDefault(require("dotenv"));
require("mocha");
const moment_1 = __importDefault(require("moment"));
const utils_general_1 = require("@energyweb/utils-general");
const ethers_1 = require("ethers");
const migrate_1 = require("../migrate");
const __1 = require("..");
const CertificateUtils_1 = require("../blockchain-facade/CertificateUtils");
describe('Issuer', () => {
    let blockchainProperties;
    dotenv_1.default.config({
        path: '.env.test'
    });
    const [web3Url] = process.env.WEB3.split(';');
    const provider = utils_general_1.getProviderWithFallback(web3Url);
    const deviceOwnerPK = '0x622d56ab7f0e75ac133722cc065260a2792bf30ea3265415fe04f3a2dba7e1ac';
    const deviceOwnerWallet = new ethers_1.Wallet(deviceOwnerPK, provider);
    const issuerPK = '0x50397ee7580b44c966c3975f561efb7b58a54febedaa68a5dc482e52fb696ae7';
    const issuerWallet = new ethers_1.Wallet(issuerPK, provider);
    let timestamp = moment_1.default().subtract(10, 'year').unix();
    const setActiveUser = (wallet) => {
        blockchainProperties.activeUser = wallet;
    };
    const createCertificationRequest = async (fromTime, toTime, deviceId, forAddress) => {
        setActiveUser(deviceOwnerWallet);
        const generationFromTime = fromTime ?? timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const generationToTime = toTime ?? timestamp;
        const device = deviceId ?? '1';
        const certificationRequest = await __1.CertificationRequest.create(generationFromTime, generationToTime, device, blockchainProperties, forAddress);
        return certificationRequest;
    };
    it('migrates Issuer and Registry', async () => {
        const registry = await migrate_1.migrateRegistry(provider, issuerPK);
        const issuer = await migrate_1.migrateIssuer(provider, issuerPK, registry.address);
        const version = await issuer.version();
        chai_1.assert.equal(version, 'v0.1');
        const registryAddress = await issuer.getRegistryAddress();
        chai_1.assert.equal(registryAddress, registry.address);
        blockchainProperties = {
            web3: provider,
            activeUser: issuerWallet,
            registry,
            issuer
        };
    });
    it('should be able to set private issuer contract', async () => {
        const privateIssuer = await migrate_1.migratePrivateIssuer(provider, issuerPK, blockchainProperties.issuer.address);
        await blockchainProperties.issuer.setPrivateIssuer(privateIssuer.address);
        chai_1.assert.equal(await blockchainProperties.issuer.getPrivateIssuerAddress(), privateIssuer.address);
    });
    it('encodes and decodes data properly', async () => {
        const generationStartTime = 1;
        const generationEndTime = 2;
        const deviceId = 'device123';
        const encodedData = CertificateUtils_1.encodeData({ generationStartTime, generationEndTime, deviceId });
        const decodedData = CertificateUtils_1.decodeData(encodedData);
        chai_1.assert.equal(generationStartTime, decodedData.generationStartTime);
        chai_1.assert.equal(generationEndTime, decodedData.generationEndTime);
        chai_1.assert.equal(deviceId, decodedData.deviceId);
    });
    it('gets all certification requests', async () => {
        await createCertificationRequest();
        await createCertificationRequest();
        const allCertificationRequests = await __1.CertificationRequest.getAll(blockchainProperties);
        chai_1.assert.equal(allCertificationRequests.length, 2);
    });
    it('user correctly requests issuance', async () => {
        setActiveUser(deviceOwnerWallet);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const device = '1';
        const certificationRequest = await createCertificationRequest(fromTime, toTime, device);
        chai_1.assert.isAbove(certificationRequest.id, -1);
        chai_1.assert.deepOwnInclude(certificationRequest, {
            deviceId: device,
            owner: deviceOwnerWallet.address,
            fromTime,
            toTime,
            approved: false
        });
    });
    it('issuer correctly approves issuance', async () => {
        const volume = ethers_1.BigNumber.from(1e9);
        let certificationRequest = await createCertificationRequest();
        setActiveUser(issuerWallet);
        const certificateId = await certificationRequest.approve(volume);
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.approved);
        chai_1.assert.isAbove(certificationRequest.issuedCertificateTokenId, 0);
        const deviceOwnerBalance = await blockchainProperties.registry.balanceOf(deviceOwnerWallet.address, certificateId);
        chai_1.assert.equal(deviceOwnerBalance.toString(), volume.toString());
    });
    it('user revokes a certificationRequest', async () => {
        setActiveUser(deviceOwnerWallet);
        let certificationRequest = await createCertificationRequest();
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isFalse(certificationRequest.approved);
        chai_1.assert.isFalse(certificationRequest.revoked);
        await certificationRequest.revoke();
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.revoked);
    });
    it('issuer should be able to revoke a certificationRequest', async () => {
        setActiveUser(issuerWallet);
        let certificationRequest = await createCertificationRequest();
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isFalse(certificationRequest.revoked);
        await certificationRequest.revoke();
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.revoked);
    });
    it('user shouldnt be able to revoke an approved certificationRequest', async () => {
        setActiveUser(deviceOwnerWallet);
        const volume = ethers_1.BigNumber.from(1e9);
        let certificationRequest = await createCertificationRequest();
        setActiveUser(issuerWallet);
        certificationRequest = await certificationRequest.sync();
        await certificationRequest.approve(volume);
        setActiveUser(deviceOwnerWallet);
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.approved);
        chai_1.assert.isFalse(certificationRequest.revoked);
        let failed = false;
        try {
            await certificationRequest.revoke();
        }
        catch (e) {
            failed = true;
        }
        chai_1.assert.isTrue(failed);
    });
    it('should request the same certificate after revoking one', async () => {
        setActiveUser(deviceOwnerWallet);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const device = '1';
        let certificationRequest = await createCertificationRequest(fromTime, toTime, device);
        certificationRequest = await certificationRequest.sync();
        await certificationRequest.revoke();
        certificationRequest = await certificationRequest.sync();
        const newCertificationRequest = await createCertificationRequest(fromTime, toTime, device);
        chai_1.assert.exists(newCertificationRequest);
    });
    it('should be able to request for other address', async () => {
        setActiveUser(deviceOwnerWallet);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const device = '1';
        const certificationRequest = await createCertificationRequest(fromTime, toTime, device, issuerWallet.address);
        chai_1.assert.isOk(certificationRequest);
    });
});
//# sourceMappingURL=Issuer.test.js.map